# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

from Products.Archetypes.public import listTypes
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.ZCatalog.Catalog import CatalogError


from Products.EnSimpleStaging.config import PROJECTNAME, GLOBALS, TOOLS, DEPENDENCIES

from StringIO import StringIO

def install_vc(self, out):
    if not hasattr(self, 'VersionRepository'):
        zvc = self.manage_addProduct['ZopeVersionControl']
        zvc.addRepository('VersionRepository')
        out.write("Created Version Repository.\n")

def install_tools(self, product, tools, out):
    addTool = self.manage_addProduct[product].manage_addTool
    for toolid, toolname in tools:
        if hasattr(self, toolid):
            self.manage_delObjects(ids=[toolid])
        addTool(toolname)
        out.write("Created tool: %s (%s).\n" % (toolid, toolname))

def setup_local_tools(self, out, path='public_website'):
    from Acquisition import aq_parent, aq_base
    from Products.Archetypes.Extensions.utils import install_archetypetool
    from Products.Archetypes.Extensions.utils import install_uidcatalog
    from Products.Archetypes.Extensions.utils import install_templates
    from Products.Archetypes.Extensions.utils import install_referenceCatalog
    public = self.restrictedTraverse(path)
    # Hack around acquisition so that tools get setup correctly
    public = aq_base(public).__of__(aq_parent(self))
    aq_base(public).ESS_INSTALLING_LOCAL_TOOLS = True
    install_uidcatalog(public, out)
    install_referenceCatalog(public, out)
    install_archetypetool(public, out)
    install_templates(public, out)
    del public.ESS_INSTALLING_LOCAL_TOOLS
    out.write('Installed uid and reference catalogs on public_website.\n')

def install_qi_products(self, out, products):
    qiTool = getToolByName(self, 'portal_quickinstaller')
    for product in products:
        qiTool.installProduct(product)
        print >> out, 'Installed dependency: %s' % product

def adapt_catalog(self, out):
    # relative_url is a PythonScript in skins/ensimplestaging/relative_url.py
    catalog = self.portal_catalog.aq_base
    try:
        catalog.addIndex('relative_url', 'FieldIndex')
        out.write('Added "relative_url" catalog index\n')
    except CatalogError, e:
        print >>out, 'could not add catalog index: %s' % e
    for metadataName in 'relative_url'.split():
        try:
            catalog.addColumn(metadataName)
            out.write('Added "%s" catalog metadata\n' % metadataName)
        except CatalogError, e:
            print >>out, 'could not add catalog column: %s' % e

def install(self):
    out = StringIO()

    install_qi_products(self, out, DEPENDENCIES)
    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)

    install_subskin(self, out, GLOBALS)
    install_vc(self, out)
    install_tools(self, PROJECTNAME, TOOLS, out)
    adapt_catalog(self, out)

    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()
