# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.exceptions import ReferenceException
from Products.Archetypes.utils import wrap_method, shasattr
from Products.Archetypes.ReferenceEngine import Reference
from Products.CMFCore.CatalogTool import CatalogTool
from OFS.ObjectManager import ObjectManager
from OFS.CopySupport import CopySource

def maybeReindex(obj):
    if isinstance(obj, CatalogTool):
        # Ignore as it has a different API for reindexObject
        return
    if getattr(obj, 'ESS_INSTALLING_LOCAL_TOOLS', False):
        # see Install.py, setup_local_tools()
        # catalogs are not yet in place
        return
    if shasattr(obj, 'notifyModified'):
        obj.notifyModified()
    if shasattr(obj, 'reindexObject'):
        obj.reindexObject(idxs=['modified'])

def delHook(self, tool, sourceObject=None, targetObject=None):
    wt = getToolByName(self, 'portal_workspaces')
    if ((sourceObject and wt._isProtectedUID(sourceObject.UID())) or
        (targetObject and wt._isProtectedUID(targetObject.UID()))):
        raise ReferenceException('This UID will come back on deployment')
    return call(self, 'delHook', tool, sourceObject, targetObject)

def _setOb(self, *args, **kw):
    ret = call(self, '_setOb', *args, **kw)
    maybeReindex(self)
    return ret

def _delOb(self, *args, **kw):
    ret = call(self, '_delOb', *args, **kw)
    maybeReindex(self)
    return ret

def _postCopy(self, *args, **kw):
    ret = call(self, '_postCopy', *args, **kw)
    maybeReindex(self)
    return ret

PATTERN = '__EnSimpleStaging_%s__'
def call(self, __name__, *args, **kw):
    return getattr(self, PATTERN % __name__)(*args, **kw)

wrap_method(Reference, 'delHook', delHook, pattern=PATTERN)

wrap_method(ObjectManager, '_setOb', _setOb, pattern=PATTERN)

wrap_method(ObjectManager, '_delOb', _delOb, pattern=PATTERN)

wrap_method(CopySource, '_postCopy', _postCopy, pattern=PATTERN)
