# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

import os, sys
from sets import Set

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Testing.ZopeTestCase.functional import Functional
from Products.PloneTestCase import PloneTestCase

for product in ('Archetypes',
                'PortalTransforms',
                'ZopeVersionControl',
                'CMFStaging',
                'EnSimpleStaging',
                'MimetypesRegistry',
                'MenuGenerator',
#                'SecureMailHost',
                ):
    PloneTestCase.installProduct(product)
del product

from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.tests.utils import makeContent
from Products.EnSimpleStaging.tests.base import TestStaging

PloneTestCase.setupPloneSite()

class TestChangeSets(TestStaging):

    def testSimpleTaggingIsReversible(self):
        self.setupStagingAreas()
        mkt = self.home.mkt
        public = self.public
        public.invokeFactory('Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')
        mkt.public.invokeFactory('Folder', id='afolder')
        mkt.saveStage(label='initial')
        index, annotLast = mkt.getLastReversibleAnnotation()

    def testChangeSetsFullDeployment(self):
        self.setupStagingAreas()
        mkt = self.home.mkt
        public = self.public
        public.invokeFactory('Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')

        getChanges = mkt.getLastMovementsAndModifications

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a docuemnt, publish stage, with a label. Should
        # catch the add for this document.
        mkt.public.invokeFactory('Document', id='doc1')
        mkt.publishStage(label='1')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 2. Create a folder, edit existing document, publish stage,
        # with a label. Should catch the folder's add and the document
        # modification.
        mkt.public.invokeFactory('Folder', id='folder1')
        mkt.public.doc1.setTitle('Document 1')
        mkt.publishStage(label='2')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',)])
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 3. Create document inside folder, remove doc1, publish stage
        # with a label. Should catch the new document's add, and the
        # old document remove plus a folder1 change.
        mkt.public.folder1.invokeFactory('Document', id='doc11')
        mkt.public.manage_delObjects(ids=['doc1'])
        mkt.publishStage(label='3')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc11')])
        self.assertEquals(cs['modifications'], [('folder1',)])
        self.assertEquals(cs['removes'], [('doc1',)])

        # 4. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and a add for
        # folder1 and doc2.
        mkt.public.manage_delObjects(ids=['folder1'])
        mkt.public.invokeFactory('Folder', id='folder1')
        mkt.public.invokeFactory('Document', id='doc2')
        mkt.publishStage(label='4')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

    def testChangeSetsPartialDeployment(self):
        self.setupStagingAreas()
        mkt = self.home.mkt
        public = self.public
        public.invokeFactory('Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')

        getChanges = mkt.getLastMovementsAndModifications

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a docuemnt, publish stage, with a label. Should
        # catch the add for this document.
        mkt.public.invokeFactory('Document', id='doc1')
        mkt.publishStage(label='1')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 2. Create a folder, edit existing document, publish stage,
        # with a label. Should catch the folder's add and the document
        # modification.
        mkt.public.invokeFactory('Folder', id='folder1')
        mkt.public.doc1.setTitle('Document 1')
        # doc is a CMFDefault document. doc.setTitle() doesn't trigger a
        # reindex
        mkt.public.doc1.reindexObject()
        mkt.publishLastChanged(label='2')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',)])
        # Should work since we reindexed doc1 above.
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 3. Create document inside folder, remove doc1, publish stage
        # with a label. Should catch the new document's add, and the
        # old document remove plus a folder1 change.
        mkt.public.folder1.invokeFactory('Document', id='doc11')
        mkt.public.manage_delObjects(ids=['doc1'])
        mkt.publishLastChanged(label='3')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc11')])
        # Next one should work if the patch to _setOb got applied.
        self.assertEquals(cs['modifications'], [('folder1',)])
        self.assertEquals(cs['removes'], [('doc1',)])

        # 4. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and a add for
        # folder1 and doc2.
        mkt.public.manage_delObjects(ids=['folder1'])
        mkt.public.invokeFactory('Folder', id='folder1')
        mkt.public.invokeFactory('Document', id='doc2')
        mkt.publishLastChanged(label='4')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for testclass in (
        TestChangeSets,
        ):
        suite.addTest(makeSuite(testclass))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
