from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent
from BTrees.OOBTree import OOBTree
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject, getToolByName
from bisect import insort
from time import time
from Products.PageTemplates.PageTemplateFile import PageTemplateFile


from config import *


import os
_www = os.path.join(os.path.dirname(__file__), 'www')

_v_menus = {}
_v_generators = {}
_v_emitters = {}

def registerGenerator(name, generator, **kwargs):
    _v_generators[name] = (generator, kwargs)

def getGenerator(name):
    return _v_generators[name]

def registerEmitter(name, emitter, **kwargs):
    _v_emitters[name] = (emitter, kwargs)

def getEmitter(name):
    return _v_emitters[name]

    
def registerMenu(name, generatorName, emitterName):
    """set a menu -> gen, em"""
    _v_menus[name] = (generatorName, emitterName)

def getMenu(name):
    """get a menu -> ((gen, kw), (em, kw))"""
    gen, em = _v_menus[name]
    return _v_generators[gen], _v_emitters[em]


class MenuTool(UniqueObject, SimpleItem):
    """Display and show registered Menus and their options.
    Allow aq of menus.
    """
    
    security=ClassSecurityInfo()

    id = TOOL_NAME

    title = TOOL_TITLE
    meta_type = TOOL_TITLE

    
    manage_options = ((
        { 'label'  : 'Menus',
          'action' : 'manage_menus',
          },
        )
        + SimpleItem.manage_options
    )
        
    manage_menus = PageTemplateFile('manage_menus', _www)


    def addMenu(self, name, generator, emitter):
        """init a menu"""
        registerMenu(name, generator, emitter)


    def getMenu(self, name, asMacro=0, **kwargs):
        """get a menu at runtime"""
        ## XXX do register emitter and make it so that it can be
        ## optionally looked up
        genFac, emFac = getMenu(name)
        em, em_args = emFac
        gen, gen_args = genFac
        em = em()
        gen = gen()

        ## We need an interface that is not feed based but rather allows
        ## generators to build NodeTrees where each node will carry a
        ## .command attr and simply pass the tree to an emitter that can
        ## do a DFS or BFS as needed. Its easier to ask questions of the
        ## tree than a stream
        em_args = em_args.copy()
        em_args.update(kwargs)
        
        gen_args = gen_args.copy()
        gen_args.update(kwargs)

        ## XXX verify this is still a problem
        context = {'portal_url': self.portal_url() } # The StringExpr
        # context code creates uncollectable junk, fake a static context for
        # now 

        tree = gen.getNodeTree(context=context, menu_tool=self, **gen_args)
        v = em.walkNodeTree(tree, menu_tool=self, **em_args)
        if asMacro == 0:
            v = em.getvalue()

        del tree
        del em
        del gen
        
        return v

    def getMenus(self):
        """return information about the menus for display"""
        return _v_menus

    def getMenuNames(self):
        """the names of the menus"""
        k = _v_menus.keys()
        k.sort()
        return k

    def getGenerators(self):
        """the generator mapping"""
        return _v_generators

    def getGenerator(self, name):
        """get a live generator by name"""
        return _v_generators[name][0]()

    ## This doesn't make as much sense as with generators, but for completeness
    def getEmitters(self):
        """the emitter mapping"""
        return _v_emitters

    def getEmitter(self, name):
        """get an emitter by name"""
        return _v_emitters[name][0]()
    
InitializeClass(MenuTool)
