from UserDict import UserDict

class Command(UserDict):
    """A command modeling a single item in the menu

    Produced By an input front end and consumed by an emitter

    Not all attributes will be populated by all generators or used
    by all emitters. Buyer beware.

    Action and Title are expected however
    """
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, **kwargs):
        UserDict.__init__(self)
        UserDict.update(self, kwargs)

    def findName(self):
        return self.get('title', self.get('action', self['id']))

    def __str__(self):
        return "Command(%r)" % self.data

    def __getattr__(self, key):
        if not key.startswith('__'):
            return UserDict.get(self, key, '')
        raise AttributeError(key)

    def __getitem__(self, key):
        return self.data.get(key, '')

    def set(self, name, value):
        self[name] = value

