from StringIO import StringIO
from Products.MenuGenerator.command import *
from Products.MenuGenerator.node import Node

class BrainJarEmitter:
    """Output html for use with http://www.brainjar.com/ menu demo

    We will have to do some fun stack unwinding here. We render each
    menu to its own buffer and then just splay them all out in any
    order, let the browser assemble them.

    """

    def __init__(self, buffer=None):
        if not buffer:
            self.buffer = StringIO()

    def getvalue(self):
        return self.buffer.getvalue()

    def walkNodeTree(self, tree, *args, **kwargs):
        self.emitter(tree)
        
    def emitter(self, tree=None):
        if not tree:
            tree = self._buildNodeTree()
        buffer = self.buffer

        nodeQueue = []
        def enqueue(node, queue=nodeQueue):
            #Get all non-toplevel nodes that have children to make menus outta them
            if node.hasChildren() and node.getDepth() != 0:
                queue.append(node)
                
        tree.bfs(enqueue)

        #Handle the special case of the top level menu
        print >>buffer, '''<div class="menuBar">'''
        for node in tree.getChildren(ordered=1):
            self.emitMenuButton(node)
        print >>buffer, '''</div>'''

        #We only ever expand a level at a time walking the nodeQueue
        for node in nodeQueue:
            #Generate a full single level menu for each node
            #The ids will graph them at runtime
            self.emitSubMenu(node)
            
    
    def emitMenuButton(self, node):
        command = node.data
        parts = self._parts(node)
        print >> self.buffer, '''<a class="menuButton" %s>%s</a>''' %(parts, command.findName())

    def emitSubMenu(self, node):
        command = node.data
        print >>self.buffer, '''<div id="%s" class="menu" onmouseover="menuMouseover(event)">''' % (command.id, )
        for child in node.getChildren(ordered=1):
                self.emitMenuItem(child)
        print >>self.buffer, '''</div>'''

    def emitMenuItem(self, node):
        command = node.data
        parts = self._parts(node)
        label = self._label(node)
        print >>self.buffer, '''<a class="menuItem" %s>%s</a>''' %(parts, label)

    def _parts(self, node):
        parts = []
        command = node.data
        
        if node.hasChildren():
            if node.getDepth() > 1:
                parts.append('''onmouseover="return menuItemMouseover(event, '%s');"''' % command.id)
            else:
                parts.append('''onmouseover="return buttonMouseover(event, '%s');"''' % command.id)

        for k, v in command.items():
                #Doesn't avoid conflicts with top stuff yet
                parts.append('''%s="%s"''' %(k, v))
                
        return ' '.join(parts)

    def _label(self, node):
        command = node.data
        if node.hasChildren():
            return '''<span class="menuItemText">%s</span><span class="menuItemArrow">&#9654;</span>''' % command.findName()
        else:
            return command.findName()
        


