from StringIO import StringIO
from Products.MenuGenerator.MenuTool import registerEmitter
from Products.MenuGenerator.command import *
from Products.MenuGenerator.node import Node
from Products.MenuGenerator.MenuTool import registerEmitter

class GazingusEmitter:
    """Output html for use with Gazingus  menu demo

    This one is simple and clean to emit with the exception that it
    needs an onload hook, css and ecma all in place for it to work
    """

    def __init__(self, buffer=None):
        if not buffer:
            self.buffer = StringIO()

        self.commands = []

        self.counter = 0
            
    def getvalue(self):
        return self.buffer.getvalue()

    __call__ = getvalue

    def walkNodeTree(self, tree, **kwargs):
        mode = kwargs.get('mode', 'menu')
        if mode == 'script':
            self.emit_script(tree, **kwargs)
        else:
            self.emit_StartMenu()
            tree.bfs(self.treeEmit, self.treeCloser, **kwargs)
            self.emit_EndMenu()
            


    def treeEmit(self, node, **kwargs):
        command = node.data
        if not command: return
        
        if node.hasChildren() and node.getDepth >= 1:
            self.emit_StartSubMenu(node, **kwargs)
        else:
            self.emit_Normal(node, **kwargs)
            
    def treeCloser(self, node, **kwargs):
        command = node.data
        if not command: return

        if node.hasChildren() and node.getDepth() >= 1:
            self.emit_EndSubMenu(node)

        
    def emit_StartMenu(self):
        print >>self.buffer, '''<!-- Start Menu -->\n
\t<div id="mainMenu">
\t\t<ul id="menuList">'''

    def emit_EndMenu(self):
        print >>self.buffer, '''\t</ul></div><!-- End Menu -->'''

    def emit_StartSubMenu(self, node, **kwargs):
        command = node.data

        body = command.findName()
        if command.recipe:
            #Funky binding, how to make this generic and better?
            try:
                body = kwargs['menu_tool'].font_tool(command.recipe, body).tag(alt=body, title=body)
            except (KeyError, AttributeError):
                pass

        img = ''
        if command.icon:
            img = '<img src="%s"/> ' % command.icon

        # we are writing our own ids and classes, so take them out of the parts dict
        abridgedCommands = command.copy()
        try: 
            del abridgedCommands['id']
        except:
            pass        
        try: 
            del abridgedCommands['class']
        except:
            pass        

        parts = self._parts(abridgedCommands)

                
        print >>self.buffer, '''\t\t<li class="menubar"> <!-- Start Sub %s -->
\t\t<a id="%sActuator" class="actuator" %s>%s%s</a>
\t\t   <ul id="%sMenu" class="menu"> <!-- SubMenu Elements -->''' % (command.findName(), command.id, parts, img, body, command.id)

        
    def emit_EndSubMenu(self, node, **kwargs):
        command = node.data
        print >>self.buffer, '''\t\t</ul> <!-- End SubMenu Elements -->
\t\t</li> <!-- End Sub %s -->''' %command.findName()

    def emit_Normal(self, node, **kwargs):
        command = node.data
        body = command.findName()

        if command.recipe:
            #Funky binding, how to make this generic and better?
            try:
                body = kwargs['menu_tool'].font_tool(command.recipe, body).tag(alt=body, title=body)
            except (KeyError, AttributeError):
                pass
            
        parts = self._parts(command)
        
        img = ''
        if command.icon:
            img = '<img src="%s"/> ' % command.icon
            
        print >>self.buffer, '''\t\t\t<li><a %s>%s%s</a></li>''' %(parts, img, body)
                    

    def _parts(self, command):
        parts = []

        #XXX rename the attr
        if command.action:
            command['href'] = command.action
            del command['action']

                    
        for k, v in command.items():
                parts.append('''%s="%s"''' %(k, v))

        return ' '.join(parts)


    def emit_script(self, tree, **kwargs):
        write = self.buffer.write
        write('''<script type="text/javascript">
        <!--
        window.onload = function() {\n''')

        def emit_Init(node, write=write):
            
            if node.hasChildren() and node.data and node.getDepth() == 1:
                command = node.data
                write('''\tinitializeMenu("%sMenu", "%sActuator");\n''' % (command.id, command.id))

        tree.bfs(emit_Init)
        
        write('''
        }
        ''')

        #XXX TODO WRITE OUT CONTEXT SEN. MENU hilight/open code

        write('''
        //-->
</script>
        ''')

registerEmitter('gazingus', GazingusEmitter)
