from StringIO import StringIO
from Products.MenuGenerator.MenuTool import registerEmitter
from Products.MenuGenerator.command import *

class HTMLEmitter:
    """This is about the simplest case, as each command is fed to the
    emitter it simply outputs the correct HTML.
    Little state needs to be maintained
    """
    def __init__(self, buffer=None):
        if not buffer:
            self.buffer = StringIO()
            
    def getvalue(self):
        return self.buffer.getvalue()

    def walkNodeTree(self, tree, **kwargs):
        tree.bfs(self.treeEmit, self.treeCloser, **kwargs)

    def treeEmit(self, node, **kwargs):
        command = node.data
        if command is None: return
        if node.hasChildren():
            self.emit_StartMenu(command, **kwargs)
        else:
            self.emit_Normal(command, **kwargs)
            

    def treeCloser(self, node, **kwargs):
        command = node.data
        if not command: return
        self.emit_EndMenu(command, **kwargs)
    
    def emit(self, command):
        emitter = getattr(self, "emit_%s" % command.type)
        emitter(command)


    def emit_Normal(self, command, **kwargs):
        cssClass = command.cssClass
        if command._menu_active:
            cssClass = "%s currentMenuItem" % (cssClass)
        elif command._menu_active_ancestor:
            cssClass = "%s currentMenuItemAncestor" % (cssClass)

        cssClass = cssClass.strip()
        if cssClass:
            print >>self.buffer, '<li class="%s">' % cssClass
        else:
            print >>self.buffer, "<li>",
            
        if command.action:
            print >>self.buffer, '<a href="%(action)s" ' % command,
            if command.anchor:
                print >>self.buffer, 'name="%(anchor)s" ' % command,
            if command.id:
                print >>self.buffer, 'id="%(id)s" ' % command,
            if command.target:
                print >>self.buffer, 'target="%(target)s" ' % command,
            print >>self.buffer, ">", 

            if command.icon:
                print >>self.buffer, '<img src="%(icon)s" border=0> ' %command,
            

        body = command.title
        if command.recipe:
            #XXX We need a general pattern for this and an expansion to recipe to handle mouseover/multi style
            if command._menu_active and command.selectedRecipe:
                body = kwargs['menu_tool'].font_tool(command.selectedRecipe, body).tag(alt=body, title=body)
            else:
                body = kwargs['menu_tool'].font_tool(command.recipe, body).tag(alt=body, title=body)
        print >>self.buffer, body,

        if command.action:
            print >>self.buffer, "</a>"

        print >>self.buffer, "</li>"

        
    def emit_StartMenu(self, command, **kwargs):
        if command.cssClass:
            print >>self.buffer, '<ul class="%(cssClass)s">' % command
        else:
            print >>self.buffer, "<ul>",

    def emit_EndMenu(self, command, **kwargs):
        print >>self.buffer, "</ul>"

    emit_StartSubMenu = emit_StartMenu
    emit_EndSubMenu = emit_EndMenu
    
        
registerEmitter('html', HTMLEmitter)
