from StringIO import StringIO
from Products.MenuGenerator.MenuTool import registerEmitter
from Products.MenuGenerator.command import *


class TreeMenuEmitter:
    """Output a tree for http://www.treemenu.com/ type trees
    """

    def walkNodeTree(self, tree, **kwargs):
        self.buffer = StringIO()
        tree.bfs(self.tag)
        print >>self.buffer, """var %s = null;\n%s = new MTMenu();""" % (tree.name, tree.name)
        tree.bfs(self.walk, self.postfix)
        
    def getvalue(self):
        return self.buffer.getvalue()
    

    def tag(self, node):
        #mark each nodes data with a counter name
        command = node.data
        if not command: return
        try: 
            command.set('menuId', self._currentVar())
        except:
            pass
        
        
    def walk(self, node):
        command = node.data
        if not command: return
        #emit the base node info
        parts = self.emit_parts(node, ( ('string', 'title', None),
                                        ('string', 'action', None),
                                        ('var', 'target', None),
                                        ('string' 'tooltip', None)
                                        ))

        indent = '' * (node.getDepth() -1)
        name = command.get('menuId')
        hasChildren  = node.hasChildren()
        parent = node.getParent()
        try:
            parentId = parent.data.get('menuId')
        except:
            if parent:
                parentId = parent.name
            else:
                parentId = node.name

        if hasChildren:
            print >>self.buffer,  "%s//submenu for %s" % (indent, command.findName())
            print >>self.buffer, """%svar %s = null;\n%s%s = new MTMenu();""" % (indent, name, indent, name)

        print >>self.buffer, "%s%s.addItem(%s);" % (indent, parentId, parts)


    def postfix(self, node):
        command = node.data
        if not command: return
        indent = '' * (node.getDepth() -1)
        name = command.get('menuId')
        hasChildren  = node.hasChildren()
        parent = node.getParent()

        try:
            parentId = parent.data.get('menuId')
        except:
            if parent:
                parentId = parent.name
            else:
                parentId = node.name
        
        if hasChildren:
            print >>self.buffer, "%s%s.makeLastSubmenu(%s);\n" % (indent, parentId, name)
        
    
    def __init__(self, buffer=None):
        if not buffer:
            self.buffer = StringIO()

        self.counter = 0
            
    
    def _currentVar(self):
        self.counter += 1
        return "menu_%02d" % self.counter
    
    def emit(self, command):
        emitter = getattr(self, "emit_%s" % command.type)
        emitter(command)


    def emit_parts(self, node, ordering, delimiter=','):
        """return a list of  elements processed in 'ordering' order and delimited from command
        while the values are in a valid range
        ordering should come in the format (type (string/var), name, default)
        """
        command = node.data
        parts = []
        for t, n, d in ordering:
            v = command.get(n, d)
            if v: 
                if t == 'string':
                    v = "'%s'" % v.replace("'", "\\'")
                elif n == "target": # Special Case
                    if v != 'null':
                        v = "'%s'" % v
            else:
                break
            parts.append(v)
        return delimiter.join(parts)
        
registerEmitter('treemenu', TreeMenuEmitter)
    

            

