from StringIO import StringIO
from Products.MenuGenerator.command import *
from Products.MenuGenerator.MenuTool import registerEmitter
from Products.MenuGenerator.node import Node

class UltimateEmitter:
    """Output html for use with UltimateMenu 

    """
    def __init__(self, buffer=None):
        if not buffer:
            self.buffer = StringIO()
            
    def getvalue(self):
        return self.buffer.getvalue()

    __call__ = getvalue


    def indent(self, node):
        print >>self.buffer, "  " * node.getDepth(), 
        
    def write(self, node, msg):
        self.indent(node)
        print >>self.buffer, msg

    def walkNodeTree(self, tree, *args, **kwargs):
        mode = kwargs.get('mode', 'menu')
        if mode == "menu":
            tree.bfs(self.treeEmit, **kwargs)
        elif mode == "top":
            self.emit_Top(tree, **kwargs)
            
            
    def treeEmit(self, node, **kwargs):
        command = node.data
        if not command: return
        
        self.emit_Normal(node, **kwargs)
        
            
    def emit_Normal(self, node, **kwargs):
        command = node.data
        depth = node.getDepth()

        #if command.get('url') and not command.get('action'):
        #    command['action'] = command['url']

        if command.get('title') and not command.get('label'):
            command['label'] = command['title']

        body = command['label']
        if command.recipe:
            #Funky binding, how to make this generic and better?
            body = kwargs['menu_tool'].font_tool(command.recipe, body).tag(alt=body, title=body)
            
        command['body'] = body

        if depth == 1:
            ## add main link item ("url","Link name",width,"text-alignment","_target","alt text",top position,left position,"key trigger","mCOLOR","rCOLOR","aLINK","aHOVER")
            self.write(node, '''MI("%(action)s", '%(body)s' , %(width)s, "%(text_alignment)s", "%(target)s", "%(alt)s", %(top_position)s, %(left_position)s , "%(key_trigger)s", "%(mCOLOR)s", "%(rCOLOR)s", "%(aLINK)s", "%(aHOVER)s");''' % command)

            # define submenu properties (width,"align to edge","text-alignment",v offset,h offset,"filter")
            self.write(node, '''SP(%(width)s, "%(align_to_edge)s", "%(text_alignment)s", %(v_offset)s, %(h_offset)s, "%(filter)s");''' % command)

        elif depth == 2:
            # add submenu link items ("url","Link name","_target","alt text")
            self.write(node, '''SI("%(action)s", '%(body)s', "%(target)s", "%(alt_text)s");''' % command)
            if node.hasChildren():
                self.write(node, """CP(%(width)s, "%(align_to_edge)s", "%(text_alignment)s", %(v_offset)s, %(h_offset)s, "%(filter)s");""" % command)
                
        elif depth == 3:
            # add child menu link items ("url","Link name","_target","alt text")
            self.write(node, """CI("%(action)s", '%(body)s', "%(target)s", "%(alt_text)s");""" % command)


    def emit_Top(self, node, **kwargs):
        """UDM positioning is odd and the top level menu should be
        static for inline placement in the page, this had the
        advantage that if JS is off you still get a menu. Coupled with
        the font tool this can still be dynamic graphical menu
        stuff"""

        #We only emit the top level nodes
        children = node.getChildren(ordered=1)
        i = 0
        first = children[0] #Used in menu alignment
        
        for child in children:
            command = child.data
            if command.get('title') and not command.get('label'):
                command['label'] = command['title']

            body = command['label']
            if command.recipe:
                #Funky binding, how to make this generic and better?
                body = kwargs['menu_tool'].font_tool(command.recipe, body).tag(alt=body, title=body)
            command['body'] = body
            command['index'] = i
            i += 1

            js = '''onmouseover="deactivateMenus()"'''
            if child.hasChildren():
                if first.data.get('alignment', 'vert').startswith('horz'):
                    #Horz menu
                    js = '''onmouseover="offsetMenu('%s', %s)"''' % (first.data['id'], command['index'])
                else:
                    #Vert menu
                    js = '''onmouseover="offsetMenu('%(id)s', %(index)s)"''' % command

            command['js'] = js
            
            self.write(node, '''<a id="%(id)s" href="%(action)s" %(js)s>%(body)s</a>''' % command)

registerEmitter('ultimate', UltimateEmitter)
