from Acquisition import aq_base
from Products.MenuGenerator.config import *
from Products.MenuGenerator.command import Command
from Products.MenuGenerator.node import Node
from Products.MenuGenerator.MenuTool import registerGenerator

# what doesn't show up in the menu
exclude = {'Members':True,
    }

class PloneNavGenerator:
    ''' generator for plone navigation based on OFSGenerator
        depth determines the number of levels to be shown
        a depth of 1 should cause it to behave like the current navigation
        path is the absolute url for where we are currently at '''
        
    def getNodeTree(self, **kwargs):
        node = Node('root')
        root = kwargs['root']
        node.data = Command(title=root.title_or_id(),
            icon=root.getIcon(),
            action=root.absolute_url())
        depth = kwargs.get('depth', 2) # determines no of levels shown
        path = kwargs.get('path', '')
        self.walk(root, node, depth, path)
        return node
    
    def walk(self, root, node, depth, path):
        contentValues = root.contentValues()
        for entry in contentValues:
            id = getattr(aq_base(entry), 'id', None)
            if not id or str(id).startswith('.') or str(id) in exclude:
                continue # a kind of private
            action = entry.absolute_url()
            if (depth > 0 or path.startswith(node.data.action)
                ) and hasattr(aq_base(entry), 'contentValues'):
                child = node.addChild(entry.getId(),
                    Command(title=entry.title_or_id(), 
                    icon=entry.getIcon(), action=action))
                self.walk(entry, child, depth-1, path)
                
                
        
registerGenerator('plone_nav', PloneNavGenerator)
