from Acquisition import aq_base
from Products.MenuGenerator.MenuTool import registerGenerator
from Products.MenuGenerator.command import Command
from Products.MenuGenerator.config import *
from Products.MenuGenerator.node import Node
from Products.MenuGenerator.utils import StringExpression
from StringIO import StringIO
from parser import MenuHandler, parseString
from xml.sax import make_parser

class XMLHandler(MenuHandler):
    #Node Handling methods
    def process_node(self, attrs):
        elementName = attrs.get('id', attrs.getValue('title'))
        n = self.node = self.node.addChild(elementName)
        d = {}
        for k in attrs.keys():
            #Prep attributes as expressions
            d[str(k)] = StringExpression(attrs.getValue(k))
        n.data = self.commandClass(**d) 


class XMLGenerator:
    def getNodeTree(self, **kwargs):
        commandClass = kwargs.get('commandClass', Command)
        context = kwargs.get('context', None)
        data = kwargs.get('data')
        
        tree = Node('menu')
        m = XMLHandler(tree, commandClass)
        parseString(data, m)
        del m
        
        #Eval an attributes in context
        def eval_attrs(node, context=context):
            command = node.data
            if not command: return
            
            for k in command.keys():
                expr = command.get(k)
                if expr and isinstance(expr, StringExpression):
                    v = expr(context)
                    #replace the expression with the value
                    #we could override str in the expr at this point too
                    command.set(k, v)

        tree.bfs(eval_attrs)

        # After we have evaluated the attributes we'd like to mark up the tree by setting an attribute on a selected node
        # and another attribute on each of its parents, this should let us do adaptive sectional highlights/expansion in a
        # slew of emitters

        # I'd like to be able to pass a callable or a URL to eval each node, when we find the matching node via this model
        # we tag up the ancestory.

        current = kwargs.get('current', '')
        menu_tool = kwargs.get('menu_tool')
        if current:
            def eval_current(node, current=current):
                command = node.data
                if not command: return
                action = command.action
                if callable(current):
                    match = current(action, menu_tool=menu_tool) #menu tool passed for aq reasons
                else:
                    match = current == action

                if match:
                    #This node seems to be the current one
                    command.set('_menu_active', 1)
                    while 1:
                        node = node.getParent()
                        if not node: break
                        if node.data:
                            node.data.set('_menu_active_ancestor', 1)
                            
            tree.bfs(eval_current)

        return tree
    
registerGenerator('xmlconfig', XMLGenerator)
