from types import ListType
from StringIO import StringIO

class Node:
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, name, data=None, parent=None):
        self.name = name
        self.children = {}
        self.data = data
        self.index = None
        self._count = 0
        self.parent = parent

    def __len__(self):
        return len(self.children)

    def __nonzero__(self):
        if self.data or self.children:
            return 1
        return 0

    def __getattr__(self, key):
        try:
            return self.children[key]
        except KeyError:
            raise AttributeError(key)



    def merge(self, otherNode):
        """Add another node to this one updating children,
        currently this destroys otherNodes children nesting and can be used for 1-shot
        additions. this is also a dumb algo, but oh well"""
        for n in otherNode.getChildren():
            n.dfs(self._mergeChildren)

    def _mergeChildren(self, node):
        """merge the node trees worrying about children"""
        path = node.getPath()[1:]
        self.addChild(path, node.data)



    def get(self, key):
        return self.children[key]

    def show(self):
        self.bfs(self.printer)

    def printer(self, node):
        depth = len(node.getPath())
        print "  " * depth, "|", node.name,
        if node.data:
            print "->", node.data
        else:
            print

    def __repr__(self):
        return "<Node %s %s>" % ('/'.join(self.getPath()), self.data)

    __str__ = __repr__

    def hasChildren(self):
        return self.children #used as a true condition

    def getChildren(self, ordered=None):
        if ordered:
            def cmpIndex(a, b):
                return cmp(a.index, b.index)
            children = self.children.values()
            children.sort(cmpIndex)
        else:
            children = self.children.items()
            children.sort()
            children = [v[1] for v in children]

        return children

    def addChild(self, path, data=None):
        if type(path) is ListType:
            path = path[:]
            n = path.pop(0)
            more = 1
        else:
            n = path
            more = 0

        node = Node(n, parent=self)
        next = self.children.setdefault(n, node)
        if not next.index:
            self._count += 1
            next.index = self._count

        if not more or not len(path):
            next.data = data
            return next

        next.addChild(path, data)
        return next

    def addChildPath(self, path, data=None):
        path = path.split('.')
        self.addChild(path, data)

    def getByPath(self, path):
        path = path.split('.')
        node = self
        for part in path:
            node = node.get(part)
        return None

    def isLeaf(self):
        return len(self.children) == 0

    def getPath(self):
        parts = [self.name]
        while 1:
            self = self.getParent()
            if not self: break
            parts.insert(0, self.name)
        return parts

    def getDepth(self):
        return len(self.getPath()) - 1 ##don't count self


    def getParent(self):
        return self.parent


    def bfs(self, visit, post_visit=None, **kwargs):
        visit(self, **kwargs)
        for child in self.getChildren(ordered=1):
            child.bfs(visit, post_visit, **kwargs)
        if post_visit: post_visit(self, **kwargs)


    def dfs(self, visit, post_visit=None, **kwargs):
        for child in self.getChildren(ordered=1):
            child.dfs(visit, post_visit, **kwargs)
        visit(self, **kwargs)


    def reset(self):
        def unlink(node):
            node.parent = None
        def kill(node):
            node.data = None
            node.children = None

        self.bfs(unlink)
        self.dfs(kill)

