""" 
Unit tests for the backend emitter code

$Id: test_emitter.py,v 1.1.1.1 2003/08/12 22:40:27 runyaga Exp $
"""

import unittest
import Zope

try:
    Zope.startup()
except: # Zope > 2.6
    pass


from rigging import *



class EmitterTests(unittest.TestCase):
    def test_htmlEmmitter(self):
        html = HTMLEmitter()

        commands = [
            Command(type=START_MENU),

            Command(action='http://foo.bar/',
                    title="Test Link",
                    type=NORMAL_MENU,
                    ),

            Command(type=END_MENU),
            ]

        
        for c in commands:
            html.emit(c)

        out = html.getvalue()
        validate(input=out, output="HTMLEmitter.out")

    def test_subMenu(self):
        html = HTMLEmitter()

        commands = [
            Command(type=START_MENU),

            Command(action='http://foo.bar/',
                    title="Test Link",
                    type=NORMAL_MENU,
                    ),

            Command(type=START_SUBMENU),

            Command(action='http://foo.com/',
                    title="Test Link2",
                    type=NORMAL_MENU,
                    ),

            Command(type=END_SUBMENU),
            
            Command(type=END_MENU),
            ]

        
        for c in commands:
            html.emit(c)

        out = html.getvalue()
        validate(input=out, output="HTMLEmitterSubmenu.out")

    def test_TreeMenu(self):
        tree = TreeMenuEmitter()

        commands = [
            Command(type=START_MENU),

            Command(action='http://foo.bar/',
                    title="Test Link",
                    type=NORMAL_MENU,
                    ),

            Command(type=START_SUBMENU),

            Command(action='http://foo.com/',
                    title="Test Link2",
                    type=NORMAL_MENU,
                    ),

            Command(type=END_SUBMENU),
            
            Command(type=END_MENU),
            ]

        
        for c in commands:
            tree.emit(c)

        out = tree.getvalue()
        validate(input=out, output="testTree.out")


    def test_brainJar(self):
        e = BrainJarEmitter()
        
        commands = [
            Command(type=START_MENU, id="foo"),
            
            Command(action='http://foo.bar/',
                    title="Test Link",
                    type=NORMAL_MENU,
                    id="foo",
                    ),

            Command(type=START_SUBMENU, id="bar"),

            Command(action='http://foo.com/',
                    title="Test Link2",
                    type=NORMAL_MENU,
                    id="bar",
                    ),

            Command(type=END_SUBMENU),
            
            Command(type=END_MENU),
            ]

        
        for c in commands:
            e.emit(c)

        assert e.getvalue()

        
        

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest( unittest.makeSuite( EmitterTests ) )
    return suite

if __name__ == '__main__':
    unittest.main( defaultTest = 'test_suite' )
