""" 
Test the menu expanded from metadata

$Id: test_metagen.py,v 1.2 2004/02/03 15:12:16 bcsaller Exp $
"""

import Zope

try:
    Zope.startup()
except: # Zope > 2.6
    pass


from Products.MenuGenerator.tests.rigging import *
from types import ListType, TupleType

from Products.MenuGenerator.emitters import *
from Products.MenuGenerator.generators.MetaGen import MetaGen, Node

LOCATIONS = ("Building1", "Building2", "Building3")
SETUPS = ("ModeA", "ModeB", "ModeC", "ModeD")
RELATIONS = ("ThingA", "ThingB", "ThingC")

class Blob:
    def __init__(self, name, loc, setup, relation):
        if type(loc) != ListType: loc = [loc]
        if type(setup) != ListType: setup = [setup]
        if type(relation) != ListType: relation = [relation]

        self.name = name
        self.loc = loc
        self.setup = setup
        self.relation = relation

    def __str__(self):
        return self.name
    
    def getLocation(self):return self.loc
    def getSetup(self): return self.setup
    def getRelation(self): return self.relation

    #Fake simpleItem iface
    def absolute_url(self): return self.name
    def title_or_id(self): return self.name
    def getIcon(self): return "document.gif"

SAMPLE_CONFIG = (
    ('Location', 'Setup', 'Relation'),
    ('Setup', 'Relation', 'Location'),
    )

a = Blob('a', LOCATIONS[0], SETUPS[0], RELATIONS[0])
b = Blob('b', LOCATIONS[1], SETUPS[1], RELATIONS[1])
c = Blob('c', LOCATIONS[0], [SETUPS[2], SETUPS[3]], RELATIONS[0])
d = Blob('d', [LOCATIONS[0], LOCATIONS[1]], SETUPS[1], [RELATIONS[0], RELATIONS[1]])
SAMPLE_DATA = (a, b, c)
SD2         = (a, b, c, d)


class MenuTests(unittest.TestCase):
    def setUp(self):
        self.meta = MetaGen()

    def test_nodes(self):
        n = Node('root')
        n.addChildPath('a.b.c', 'foo')
        assert n.a.b.c.data == 'foo'

        #now add a dup node b with child d and
        #make sure c is still a child
        n.addChildPath('a.b.d', 'bar')
        assert n.a.b.c.data == 'foo'
        assert n.a.b.d.data == 'bar'
        assert n.a.b.d.getPath() == ['root', 'a', 'b', 'd']
        
    def test_treebuilder(self):
        #given sample data make sure we can load nodes into a tree with mkdir -p like sementics
        m = self.meta
        tree = m._buildTree(SAMPLE_CONFIG, SAMPLE_DATA)
        assert tree.get(LOCATIONS[0]).get(SETUPS[0]).get(RELATIONS[0]).get(a.name).data == a
        assert tree.get(LOCATIONS[1]).get(SETUPS[1]).get(RELATIONS[1]).get(b.name).data == b
        assert tree.get(LOCATIONS[0]).get(SETUPS[2]).get(RELATIONS[0]).get(c.name).data == c
        assert tree.get(LOCATIONS[0]).get(SETUPS[3]).get(RELATIONS[0]).get(c.name).data == c
        assert tree.get(SETUPS[3]).get(RELATIONS[0]).get(LOCATIONS[0]).get(c.name).data == c
        assert tree.get(SETUPS[2]).get(RELATIONS[0]).get(LOCATIONS[0]).get(c.name).data == c
        #woo!

    def test_treebuilder2(self):
        # loading extra data doesn't break the existing tree
        m = self.meta
        tree = m._buildTree(SAMPLE_CONFIG, SD2) # Note SD2
        assert tree.get(LOCATIONS[0]).get(SETUPS[0]).get(RELATIONS[0]).get(a.name).data == a
        assert tree.get(LOCATIONS[1]).get(SETUPS[1]).get(RELATIONS[1]).get(b.name).data == b
        assert tree.get(LOCATIONS[0]).get(SETUPS[2]).get(RELATIONS[0]).get(c.name).data == c
        assert tree.get(LOCATIONS[0]).get(SETUPS[3]).get(RELATIONS[0]).get(c.name).data == c
        assert tree.get(SETUPS[3]).get(RELATIONS[0]).get(LOCATIONS[0]).get(c.name).data == c
        assert tree.get(SETUPS[2]).get(RELATIONS[0]).get(LOCATIONS[0]).get(c.name).data == c
        #woo!

    def test_generator(self):
        #walk a built tree and emit MenuGenerator commands
        m = self.meta
        em  = XMLEmitter()
        tree = m.getNodeTree(data=SAMPLE_DATA, config=SAMPLE_CONFIG)
        text  = em.walkNodeTree(tree)
        print text
        #validate(text, "metagen.out")
        

    
        

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest( unittest.makeSuite(MenuTests ) )
    return suite

if __name__ == '__main__':
    unittest.main( defaultTest = 'test_suite' )
