""" 
Test the menu expanded from metadata

$Id: test_node.py,v 1.2 2003/11/26 18:28:20 bcsaller Exp $
"""

import Zope

try:
    Zope.startup()
except: # Zope > 2.6
    pass


from Products.MenuGenerator.tests.rigging import *
from types import ListType, TupleType

from Products.MenuGenerator.node import Node

class NodeTests(unittest.TestCase):
    def test_nodes(self):
        n = Node('root')
        n.addChildPath('a.b.c', 'foo')
        assert n.a.b.c.data == 'foo'

        #now add a dup node b with child d and
        #make sure c is still a child
        n.addChildPath('a.b.d', 'bar')
        assert n.a.b.c.data == 'foo'
        assert n.a.b.d.data == 'bar'
        assert n.a.b.d.getPath() == ['root', 'a', 'b', 'd']
        

    def test_ordering(self):
        n = Node('root')
        n.addChildPath('a.b.c', 'foo')
        # Add something lexagraphically preciding that node
        n.addChildPath('a.a.c', 'bar')
        n.addChildPath('a.b.a', 'first')
        n.addChildPath('a.b.aa', 'second')

        sortedChildren = n.a.b.getChildren()
        orderedChildren = n.a.b.getChildren(ordered=1)

        sData = [c.data for c in sortedChildren]
        oData = [c.data for c in orderedChildren]
        assert sData == ['first', 'second', 'foo']
        assert oData == ['foo', 'first', 'second']

    def test_walk(self):
        n = Node('root')
        n.addChildPath('a.b.c', None)
        n.addChildPath('a.b.d', None)
        n.addChildPath('a.e.f', None)
        #   A
        # B   E
        #C D F
        def show(node):
            print node.name,

        #print "BFS", n.bfs(show)
        #print
        #print "DFS", n.dfs(show)

    def test_add(self):
        n = Node('root')
        n.addChildPath('a.b.c', '1')
        n.addChildPath('a.b.d', '2')
        n.addChildPath('a.e.f', '3')

        r = Node('root')
        r.addChildPath('d.e.f', '4')
        r.addChildPath('a.b.d', '5')
        r.addChildPath('w.q.l', '6')

        n.merge(r)
        assert n.a.b.c.data == '1'
        assert n.a.b.d.data == '5'
        assert n.w.q.l.data == '6'
        assert n.a.e.f.data == '3'
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(NodeTests))
    return suite

if __name__ == '__main__':
    unittest.main( defaultTest = 'test_suite' )
