import re
import sys

_var = re.compile(r'\${([^}]*?)}')
_marker = []

class PathExpression:
    """A / delimited path of addressable objects"""
    def __init__(self, text):
        self.text = text
        self.path = text.split('/')

    def __call__(self, ns):
        def walk(object, target):
            #Try some magic to get a value for target from object
            v = getattr(object, target, None)
            if v: return v

            try:
                v = object[target]
            except (TypeError, KeyError):
                pass
            else:
                return v
            
            try:
                object.get(target)
            except (AttributeError, KeyError):
                pass
            else:
                return v

            raise AttributeError("Can't evaluate %r from %r" %(target, object))
            
            
        object = ns
        for part in self.path:
            object = walk(object, part)
            if callable(object):
                object = object()
            
        if object != ns:
            return object
        
        return None

class StringExpression:
    def __init__(self, text):
        self.text = text
        # if the re matches even once we need to
        # build out the expression for eval
        # if not we fast_path it for this instance
        m = _var.search(self.text)
        self._fast = 0
        if not m:
            self._fast = 1
        else:
            self._compile()

    def __str__(self):
        return self.__call__()
    
    def __call__(self, ns=None):
        #Run the program using ns
        if self._fast:
            return self.text
        output = []
        if not ns:
            frame = sys._getframe()
            context = _marker
            while context is _marker and frame is not None:
                context = frame.f_locals.get('econtext', _marker)
                frame = frame.f_back
            if context is _marker:
                del frame
                raise RuntimeError, 'Context not found'
            del frame
            
            #For a stock eval to work we have to collapse the multimapping
            ns = context.global_vars
            del context

        for type, data in self._program:
            if type == 'text':
                output.append(data)
            elif type == 'code':
                v = eval(data, ns)
                output.append(v)
            elif type == 'path':
                v = data(ns)
                output.append(v)
        del ns
        return ''.join(output)

    def _compile(self):
        # self._program = ( (type, data), ...)
        self._program = []
        chunks = _var.split(self.text)
        for i in range(len(chunks)):
            ctype = (i % 2 == 0) and 'text' or 'code'
            data = chunks[i]
            if ctype == 'code':
                #do we upgrade to a path expr?
                if '/' in data:
                    ctype = 'path'
                    data = PathExpression(data)
                    
            self._program.append((ctype, data))
        
        
