##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

import os
import logging
from eed.logger import TRACE
from eed.common import CommandManager

class Command:

    def __init__(self, command):
        self.command = command

    def execute(self):
        return os.system(self.command)

    def add(self, *args, **kw):
        return self.execute()

    update = remove = add

class CommandQueue(list):

    def __init__(self, name):
        self.name = name
        self.resource_name = 'CommandQueue'

    def _manager(self):
        from eed.txn import get_resource
        manager = get_resource(self.resource_name, CommandManager)
        return manager
    manager = property(_manager)

    def log(self, msg):
        logger = logging.getLogger('eed.%s' % self.name)
        logger.log(TRACE, msg)

    def execute(self):
        for command in self:
            self.log('Adding %r to CommandManager' % command)
            command.execute()
            self.manager.append(command)

    def cleanup(self):
        self.log('Cleaning up')
        while self:
            self.pop()

    def queue(self, command):
        if not command in self:
            self.log('Adding %r to queue' % command)
            self.append(command)

after_hooks_queue = CommandQueue('after_hooks_queue')
before_hooks_queue = CommandQueue('before_hooks_queue')
