##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

import os
import logging
from logging import WARNING, INFO, DEBUG

# Values defined in ZODB.loglevels.
BLATHER = 15
TRACE = 5

_pid = str(os.getpid())
logger = None

def setupLogging(level=TRACE):
    global logger
    if logger is not None:
        # Already setup
        return
    # Configure logger for eed.
    logger = logging.getLogger('eed')
    logger.setLevel(level)
    fmt = logging.Formatter(
        "------\n%(asctime)s %(levelname)s %(name)s %(message)s",
        "%Y-%m-%dT%H:%M:%S")
    handler = logging.StreamHandler()
    handler.setLevel(level)
    handler.setFormatter(fmt)
    logger.addHandler(handler)

    # Now, configure logger for ZEO.zrpc
    zeolog = logging.getLogger('ZEO.zrpc')
    zeolog.setLevel(level)
    zeolog.addHandler(handler)

def log(msg, level=INFO, exc_info=False):
    """Internal: generic logging function.
    """
    if logger is None:
        return
    message = "(%s) %s" % (_pid, msg)
    logger.log(level, message, exc_info=exc_info)

setupLogging()
