##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

__metaclass__ = type

from sqlobject import *
from datetime import datetime
from eed.common import ConnectionDescriptor

class Meta(SQLObject):

    _connection = ConnectionDescriptor()
    _cacheValues = False
    _connection_string = None

    _columns = [
        IntCol('path_hash', notNull=1),
        StringCol('path', length=255, notNull=1,
                  validator=validators.String(strip_spaces=1)),
        DateTimeCol('modified', notNull=1, default=datetime.utcnow),
        ]


class CMFMeta(SQLObject):

    _connection = ConnectionDescriptor()
    _cacheValues = False
    _connection_string = None

    _columns = [
        IntCol('path_hash', notNull=1),
        StringCol('path', length=255, notNull=1,
                  validator=validators.String(strip_spaces=1)),
        StringCol('uid', length=255, notNull=1,
                  validator=validators.String(strip_spaces=1)),
        DateTimeCol('modified', notNull=1,
                    default=datetime.utcnow),
        DateTimeCol('created', notNull=1,
                    default=datetime.utcnow),
        StringCol('portal_type', length=255,
                  notNull=0, default='',
                  validator=validators.String(strip_spaces=1)),
        StringCol('meta_type', length=255,
                  notNull=0, default='',
                  validator=validators.String(strip_spaces=1)),
        StringCol('review_state', length=255,
                  notNull=0, default='',
                  validator=validators.String(strip_spaces=1)),
        StringCol('title', length=255,
                  notNull=0, default='',
                  validator=validators.String(strip_spaces=1)),
        StringCol('description', length=255,
                  notNull=0, default='',
                  validator=validators.String(strip_spaces=1)),
        ]


class SQLObjectMeta:

    # Correct values will be stitched in by ZConfig
    name = 'meta-storage'
    impl = None

    def add(self, key, info):
        data = info.get('data', {})
        impl = self.impl
        c_keys = impl._SO_columnDict.keys()
        res = impl.select(impl.q.path == key)
        meta = {'path':key}
        for k in c_keys:
            if data.has_key(k):
                meta[k] = data[k]
            elif k in ('path_hash',):
                meta[k] = hash(key)
        count = res.count()
        if not count:
            ob = impl(**meta)
        elif count == 1:
            ob = iter(res).next()
            # XXX Seems like there's a bug with _lazyUpdate and
            # setting attributes on existing instances. Maybe some
            # caching issue?
            s = ob._lazyUpdate
            ob._lazyUpdate = False
            ob.set(**meta)
            ob._lazyUpdate = s
        else:
            raise ValueError, 'Duplicate values for key %s' % key

    # Alias update to add.
    update = add

    def remove(self, key, info):
        impl = self.impl
        res = impl.select(impl.q.path == key)
        count = res.count()
        if not count:
            # do nothing, no value exists for key.
            pass
        elif count == 1:
            ob = iter(res).next()
            ob.destroySelf()
        else:
            raise ValueError, 'Duplicate values for key %s' % key

__all__ = ('SQLObjectMeta', 'CMFMeta', 'Meta')
