##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

import logging
from eed.utils import caller
from eed.logger import TRACE, DEBUG
from eed.exceptions import DeploymentInProgress
from transaction import TransactionManager

VERBOSE = TRACE - 5

def log(self, msg, level=VERBOSE):
    logger = logging.getLogger('eed.txn')
    logger.log(level, '%s: %s' % (msg, caller(3, 7)))

class DeploymentTransactionManager(TransactionManager):

    log = log

    def begin(self):
        self.log('Calling TM.begin()')
        # There can be only one transaction active at a time on the
        # deployment server, so what we do is to raise a transaction
        # here instead of just blindly aborting.
        if self._txn is not None:
            raise DeploymentInProgress, ('There is an active deployment '
                                         'in progress currently.')
        if self._synchs:
            # Synchronizers from an early transaction are registered.
            raise DeploymentInProgress, ('Synchronizers from an earlier '
                                         'transaction are still registered.')
        return super(DeploymentTransactionManager, self).begin()

    def get(self):
        self.log('Calling TM.get()')
        return super(DeploymentTransactionManager, self).get()

    def commit(self, sub=False):
        self.log('Calling TM.commit()')
        return super(DeploymentTransactionManager, self).commit(sub=sub)

    def abort(self, sub=False):
        self.log('Calling TM.abort()')
        return super(DeploymentTransactionManager, self).abort(sub=sub)

    def free(self, txn):
        self.log('Calling TM.free()')
        # Free synchronizers
        self._synchs.map(self._synchs.remove)
        return super(DeploymentTransactionManager, self).free(txn)

manager = DeploymentTransactionManager()

def get():
    return manager.get()

def begin():
    return manager.begin()

def commit(sub=False):
    manager.get().commit(sub)

def abort(sub=False):
    manager.get().abort(sub)

def free(txn):
    manager.free(txn)

def active():
    return manager._txn is not None

key = '__resources__'
def get_resources(create=False):
    if not active():
        if not create:
            return {}
        else:
            log('Implicitly creating transaction', level=TRACE)
    txn = get()
    if not hasattr(txn, key):
        setattr(txn, key, {})
    resources = getattr(txn, key)
    return resources

def add_resource(key, resource):
    get().join(resource)
    get_resources()[key] = resource

def get_resource(key, factory=None, create=False):
    rsrc = get_resources(create=create).get(key)
    if rsrc is None and factory is not None:
        rsrc = factory()
        add_resource(key, rsrc)
    return rsrc

def clear_resources():
    get_resources().clear()

# TODO: Issue deprecation warning if this variant is used?
get_transaction = get
