##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""
import types
import os
import os.path

def copy_file(infile, outfile):
    """ Read binary data from infile and write it to outfile. infile
    and outfile may be strings, in which case a file with that name is
    opened, or filehandles, in which case they are accessed directly.
    """

    if type(infile) is types.StringType:
        instream = open(infile, 'rb')
        read_in = instream.read
        close_in = instream.close
        seek_in = instream.seek
    elif type(infile) is types.IntType:
        # Maybe a file descriptor?
        def read(bytes):
            return os.read(infile, bytes)
        read_in = read
        def close():
            return os.close(infile)
        close_in = close
        def seek(pos, how):
            return os.lseek(infile, pos, how)
        seek_in = seek
    else:
        close_in = 0
        instream = infile
        read_in = instream.read
        seek_in = instream.seek

    if type(outfile) is types.StringType:
        outstream = open(outfile, 'wb')
        write_out = outstream.write
        close_out = outstream.close
    elif type(outfile) is types.IntType:
        # Maybe a file descriptor?
        def write(bytes):
            return os.write(outfile, bytes)
        write_out = write
        def close():
            return os.close(outfile)
        close_out = close
    else:
        close_out = 0
        outstream = outfile
        write_out = outstream.write

    try:
        blocksize = 2 << 16
        block = read_in(blocksize)
        write_out(block)
        while len(block) == blocksize:
            block = read_in(blocksize)
            write_out(block)
        seek_in(0)
    finally:
        if close_in: close_in()
        if close_out: close_out()

def make_target_path(originals, target):
    path = os.path.splitdrive(target)[-1]
    path = path.strip(os.path.sep)
    parts = os.path.split(path)
    destination = os.path.join(originals, *parts)
    return destination

def caller(start=1, end=1):
    """Return the name of the calling function n levels up in the frame stack.
    Ex: caller(0) ==> 'caller'; def f(): return caller(); f() ==> 'f'
    """
    retval = []
    import inspect
    for f in inspect.getouterframes(inspect.currentframe())[start:end]:
        retval.append(f[3])
        del f
    retval.reverse()
    return ' . '.join(retval)
