##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

import os
import sys
from os.path import dirname, basename, join
import unittest
from glob import glob
import tempfile
import shutil
import doctest
from doctest import DocFileSuite
from eed import txn
from eed.config import setConfiguration

me = sys.modules[__name__]
paths = dict(src_path=None,
             originals_path=None,
             tmp_path=None)

def tearDown(test):
    txn.abort()
    txn.free(txn.get())
    if os.path.exists('/tmp/test.sqlite'):
        os.remove('/tmp/test.sqlite')
    for name in ('src_path', 'tmp_path', 'originals_path'):
        d = paths[name]
        if os.path.exists(d):
            shutil.rmtree(d)
    setConfiguration(None)

def setUp(test):
    setConfiguration(None)
    txn.begin()
    paths['src_path'] = tempfile.mkdtemp()
    paths['originals_path'] = tempfile.mkdtemp()
    paths['tmp_path'] = tempfile.mkdtemp()

def test_suite():
    suite = unittest.TestSuite()
    prefix = dirname(__file__)
    files = [basename(f) for f in glob(join(prefix, '*.txt'))]

    kw = {'package': me,
          'optionflags': (doctest.ELLIPSIS
                          | doctest.REPORT_NDIFF
                          | doctest.NORMALIZE_WHITESPACE),
          'setUp':setUp,
          'tearDown': tearDown,
          'globs': {'paths':paths}
          }

    suite.addTest(DocFileSuite(*files, **kw))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
